@@ -25,7 +25,11 @@
 #include <gst/gst.h>
 
 #ifndef GST_NET_API
-#define GST_NET_API GST_EXPORT
+#ifdef BUILDING_GST_NET
+#define GST_NET_API GST_API_EXPORT        /* from config.h */
+#else
+#define GST_NET_API GST_API_IMPORT
+#endif
 #endif
 
 #endif /* __GST_NET_PRELUDE_H__ */
